@@ -8,22 +8,39 @@ import com.fasterxml.jackson.core.*;
 import com.fasterxml.jackson.core.io.SegmentedStringWriter;
 import com.fasterxml.jackson.core.util.ByteArrayBuilder;
 import com.fasterxml.jackson.core.util.Instantiatable;
+import com.fasterxml.jackson.jr.ob.api.CollectionBuilder;
+import com.fasterxml.jackson.jr.ob.api.MapBuilder;
+import com.fasterxml.jackson.jr.ob.api.ReaderWriterProvider;
 import com.fasterxml.jackson.jr.ob.comp.CollectionComposer;
 import com.fasterxml.jackson.jr.ob.comp.ComposerBase;
 import com.fasterxml.jackson.jr.ob.comp.MapComposer;
 import com.fasterxml.jackson.jr.ob.impl.*;
 
 /**
- * Main entry point for functionality.
+ * Main entry point for functionality for reading and writing JSON
+ * and configuring details of reading and writing.
  *<p>
  * Note that instances are fully immutable, and thereby thread-safe.
+ *<p>
+ * Note on source types: source to read is declared as {@link java.lang.Object}
+ * but covers following types:
+ *<ul>
+ * <li>{@link InputStream}</li>
+ * <li>{@link Reader}</li>
+ * <li>{@code byte[]}</li>
+ * <li>{@code char[]}</li>
+ * <li>{@link String}/{@link CharSequence}</li>
+ * <li>{@link URL}</li>
+ * <li>{@link File}</li>
+ * </ul>
+ * 
  */
 @SuppressWarnings("resource")
 public class JSON implements Versioned
 {
     /**
-     * Simple on/off (enabled/disabled) features for {@link JSON}; used for simple configuration
-     * aspects.
+     * Simple on/off (enabled/disabled) features for {@link JSON}; used for simple
+     * configuration aspects.
      */
     public enum Feature
     {
@@ -72,7 +89,7 @@ public class JSON implements Versioned
        /**
         * This feature can be used to indicate that the reader should preserve
         * order of the properties same as what input document has.
-        * Note that it is up to {@link com.fasterxml.jackson.jr.ob.impl.MapBuilder}
+        * Note that it is up to {@link com.fasterxml.jackson.jr.ob.api.MapBuilder}
         * to support this feature; custom implementations may ignore the setting.
         *<p>
         * Default setting is <code>true</code>, meaning that reader is expected to try to
@@ -250,13 +267,12 @@ public class JSON implements Versioned
         * Feature that enables use of public fields instead of setters and getters,
         * in cases where no setter/getter is available.
         *<p>
-        * Feature is disabled by default (for backwards compatibility), so fields
-        * are not used unless explicitly enabled.
+        * Feature is <b>enabled</b> by default since 2.10 (but was <b>disabled</b> for
+        * 2.8 and 2.9), so public fields are discovered by default.
         *
-        * @since 2.8
+        * @since 2.8 (enabled by default since 2.10)
         */
-       USE_FIELDS(false, true),
-       
+       USE_FIELDS(true, true),
        ;
 
        /*
@@ -331,6 +347,8 @@ public class JSON implements Versioned
     // Important: has to come before 'std' instance, since it refers to it
     private final static int DEFAULT_FEATURES = Feature.defaults();
 
+    public final static int CACHE_FLAGS = Feature.cacheBreakers();
+
     /**
      * Singleton instance with standard, default configuration.
      * May be used with direct references like:
@@ -365,11 +383,11 @@ public class JSON implements Versioned
     protected final JSONReader _reader;
 
     /**
-     * Blueprint isntance of the writer to use for writing JSON given
+     * Blueprint instance of the writer to use for writing JSON given
      * simple Objects.
      */
     protected final JSONWriter _writer;
-    
+
     /*
     /**********************************************************************
     /* Configuration, simple settings
@@ -387,15 +405,13 @@ public class JSON implements Versioned
      */
 
     public JSON() {
-        this(DEFAULT_FEATURES, new JsonFactory(), null);
+        this(DEFAULT_FEATURES, new JsonFactory(), null,
+                null, null, null); // reader / writer / pretty-printer
     }
 
-    protected JSON(int features,
-            JsonFactory jsonF, TreeCodec trees)
-    {
-        this(features, jsonF, trees,
-                null, null, // reader, writer
-                null);
+    public JSON(JsonFactory jsonF) {
+        this(DEFAULT_FEATURES, jsonF, null,
+                null, null, null); // reader / writer / pretty-printer
     }
 
     protected JSON(int features,
@@ -406,23 +422,18 @@ public class JSON implements Versioned
         _features = features;
         _jsonFactory = jsonF;
         _treeCodec = trees;
-        TypeDetector td = _defaultTypeDetector(features);
-        _reader = (r == null) ? _defaultReader(features, trees, td) : r;
-        _writer = (w == null) ? _defaultWriter(features, trees, td) : w;
+        _reader = (r != null) ? r : _defaultReader(features, trees, null);
+        _writer = (w != null) ? w : _defaultWriter(features, trees, null);
         _prettyPrinter = pp;
     }
 
-    protected TypeDetector _defaultTypeDetector(int features) {
-        return TypeDetector.blueprint(features);
-    }
-
-    protected JSONReader _defaultReader(int features, TreeCodec tc, TypeDetector td) {
-        return new JSONReader(features, td, tc,
+    protected JSONReader _defaultReader(int features, TreeCodec tc, ReaderWriterProvider rwp) {
+        return new JSONReader(features, ValueReaderLocator.blueprint(features, rwp), tc,
                 CollectionBuilder.defaultImpl(), MapBuilder.defaultImpl());
     }
 
-    protected JSONWriter _defaultWriter(int features, TreeCodec tc, TypeDetector td) {
-        return new JSONWriter(features, td, tc);
+    protected JSONWriter _defaultWriter(int features, TreeCodec tc, ReaderWriterProvider rwp) {
+        return new JSONWriter(features, ValueWriterLocator.blueprint(features, rwp), tc);
     }
 
     /*
@@ -541,6 +552,22 @@ public class JSON implements Versioned
         return _with(_features, _jsonFactory, _treeCodec,
                 r, _writer, _prettyPrinter);
     }
+
+    /**
+     * Mutant factory for constructing an instance with specified {@link ReaderWriterProvider},
+     * and returning new instance (or, if there would be no change, this instance).
+     *
+     * @since 2.10
+     */
+    public JSON with(ReaderWriterProvider rwp) {
+        JSONReader r = _reader.with(rwp);
+        JSONWriter w = _writer.with(rwp);
+        if ((r == _reader) && (w == _writer))  {
+            return this;
+        }
+        return _with(_features, _jsonFactory, _treeCodec,
+                r, w, _prettyPrinter);
+    }
     
     /**
      * Mutant factory for constructing an instance with specified feature
@@ -593,8 +620,14 @@ public class JSON implements Versioned
         if (_features == features) {
             return this;
         }
+
+        // 07-Jun-2019, tatu: May need to force clearing of state if cache-afflicted
+        //    changes
+        JSONReader r = _reader.withCacheCheck(features);
+        JSONWriter w = _writer.withCacheCheck(features);
+        
         return _with(features, _jsonFactory, _treeCodec,
-                _reader, _writer, _prettyPrinter);
+                r, w, _prettyPrinter);
     }
 
     /*
@@ -603,7 +636,7 @@ public class JSON implements Versioned
     /**********************************************************************
      */
     
-    protected final JSON _with(int features,
+    protected JSON _with(int features,
             JsonFactory jsonF, TreeCodec trees,
             JSONReader reader, JSONWriter writer,
             PrettyPrinter pp)
@@ -632,6 +665,34 @@ public class JSON implements Versioned
         return (f.mask() & _features) != 0;
     }
     
+    /*
+    /**********************************************************************
+    /* Public factory methods for parsers, generators
+    /**********************************************************************
+     */
+
+    /**
+     * Factory method for opening a {@link JsonParser} to read content from one of
+     * following supported sources
+     *<ul>
+     * <li>{@link InputStream}</li>
+     * <li>{@link Reader}</li>
+     * <li>{@code byte[]}</li>
+     * <li>{@code char[]}</li>
+     * <li>{@link String}/{@link CharSequence}</li>
+     * <li>{@link URL}</li>
+     * <li>{@link File}</li>
+     * </ul>
+     *<p>
+     * Rules regarding closing of the underlying source follow rules
+     * that {@link JsonFactory} has for its {@code createParser} method.
+     *
+     * @since 2.10
+     */
+    public JsonParser createParser(Object source) throws IOException, JSONObjectException {
+        return _parser(source);
+    }
+
     /*
     /**********************************************************************
     /* API: writing Simple objects as JSON
@@ -837,22 +898,49 @@ public class JSON implements Versioned
     }
 
     @SuppressWarnings("unchecked")
-    public <T> Map<T,Object> mapFrom(Object source) throws IOException, JSONObjectException
+    public Map<String,Object> mapFrom(Object source) throws IOException, JSONObjectException
     {
         if (source instanceof JsonParser) {
             JsonParser p = _initForReading((JsonParser) source);
-            Map<Object,Object> result = _readerForOperation(p).readMap();
+            Map<?,?> result = _readerForOperation(p).readMap();
             p.clearCurrentToken();
-            return (Map<T,Object>) result;
+            return (Map<String,Object>) result;
         }
         JsonParser p = _parser(source);
         try {
             _initForReading(_config(p));
-            Map<Object,Object> result = _readerForOperation(p).readMap();
+            Map<?,?> result = _readerForOperation(p).readMap();
             JsonParser p0 = p;
             p = null;
             _close(p0, null);
-            return (Map<T,Object>) result;
+            return (Map<String,Object>) result;
+        } catch (Exception e) {
+            return _closeWithError(p, e);
+        }
+    }
+
+    /**
+     * Read method for reading a {@link Map} of {@code type} (usually POJO) values.
+     *
+     * @since 2.10
+     */
+    @SuppressWarnings("unchecked")
+    public <T> Map<String,T> mapOfFrom(Class<T> type, Object source) throws IOException, JSONObjectException
+    {
+        if (source instanceof JsonParser) {
+            JsonParser p = _initForReading((JsonParser) source);
+            Map<?,?> result = _readerForOperation(p).readMapOf(type);
+            p.clearCurrentToken();
+            return (Map<String,T>) result;
+        }
+        JsonParser p = _parser(source);
+        try {
+            _initForReading(_config(p));
+            Map<?,?> result = _readerForOperation(p).readMapOf(type);
+            JsonParser p0 = p;
+            p = null;
+            _close(p0, null);
+            return (Map<String,T>) result;
         } catch (Exception e) {
             return _closeWithError(p, e);
         }
@@ -926,7 +1014,7 @@ public class JSON implements Versioned
      * @since 2.8
      */
     @SuppressWarnings("unchecked")
-    public <T extends TreeNode> TreeNode treeFrom(Object source)
+    public <T extends TreeNode> T treeFrom(Object source)
             throws IOException, JSONObjectException
     {
         if (_treeCodec == null) {
@@ -952,6 +1040,91 @@ public class JSON implements Versioned
         }
     }
 
+    /*
+    /**********************************************************************
+    /* API: reading sequence of JSON values (LD-JSON and like)
+    /**********************************************************************
+     */
+
+    /**
+     * Method for creating {@link ValueIterator} for reading
+     * <a href="https://en.wikipedia.org/wiki/JSON_streaming">streaming JSON</a>
+     * content (specifically line-delimited and concatenated variants);
+     * individual values are bound to specific Bean (POJO) type.
+     *
+     * @since 2.10
+     */
+    public <T> ValueIterator<T> beanSequenceFrom(Class<T> type, Object source)
+        throws IOException, JSONObjectException
+    {
+        JsonParser p;
+        final boolean managed = !(source instanceof JsonParser);
+
+        if (managed) {
+            p = _parser(source);
+        } else {
+            p = (JsonParser) source;
+        }
+        p = _initForReading(_config(p));
+        JSONReader reader = _readerForOperation(p);
+        return new ValueIterator<T>(ValueIterator.MODE_BEAN, type,
+                p, reader, _treeCodec, managed);
+    }
+
+    /**
+     * Method for creating {@link ValueIterator} for reading
+     * <a href="https://en.wikipedia.org/wiki/JSON_streaming">streaming JSON</a>
+     * content (specifically line-delimited and concatenated variants);
+     * individual values are bound as "Simple" type: {@link java.util.Map},
+     * {@link java.util.List}, {@link String}, {@link Number} or {@link Boolean}.
+     *
+     * @since 2.10
+     */
+    public ValueIterator<Object> anySequenceFrom(Object source)
+        throws IOException, JSONObjectException
+    {
+        JsonParser p;
+        final boolean managed = !(source instanceof JsonParser);
+
+        if (managed) {
+            p = _parser(source);
+        } else {
+            p = (JsonParser) source;
+        }
+        p = _initForReading(_config(p));
+        JSONReader reader = _readerForOperation(p);
+        return new ValueIterator<Object>(ValueIterator.MODE_ANY, Object.class,
+                p, reader, _treeCodec, managed);
+    }
+
+    /**
+     * Method for creating {@link ValueIterator} for reading
+     * <a href="https://en.wikipedia.org/wiki/JSON_streaming">streaming JSON</a>
+     * content (specifically line-delimited and concatenated variants);
+     * individual values are bound as JSON Trees(of type that configured
+     * {@link TreeCodec}, see {@link #with(TreeCodec)}) supports.
+     */
+    public <T extends TreeNode> ValueIterator<T> treeSequenceFrom(Object source)
+        throws IOException, JSONObjectException
+    {
+        if (_treeCodec == null) {
+             _noTreeCodec("read TreeNode");
+        }
+
+        JsonParser p;
+        final boolean managed = !(source instanceof JsonParser);
+
+        if (managed) {
+            p = _parser(source);
+        } else {
+            p = (JsonParser) source;
+        }
+        p = _initForReading(_config(p));
+        JSONReader reader = _readerForOperation(p);
+        return new ValueIterator<T>(ValueIterator.MODE_TREE, TreeNode.class,
+                p, reader, _treeCodec, managed);
+    }
+
     /*
     /**********************************************************************
     /* API: TreeNode construction
