@@ -4,6 +4,8 @@ import java.io.IOException;
 
 import com.fasterxml.jackson.core.*;
 import com.fasterxml.jackson.jr.ob.JSONObjectException;
+import com.fasterxml.jackson.jr.ob.api.CollectionBuilder;
+import com.fasterxml.jackson.jr.ob.api.ValueReader;
 
 /**
  * Reader for typed Array values.
@@ -13,8 +15,9 @@ public class ArrayReader extends ValueReader
     protected final Class<?> _elementType;
     protected final ValueReader _valueReader;
 
-    public ArrayReader(Class<?> t, ValueReader vr) {
-        _elementType = t;
+    public ArrayReader(Class<?> arrayType, Class<?> elementType, ValueReader vr) {
+        super(arrayType);
+        _elementType = elementType;
         _valueReader = vr;
     }
 
@@ -24,7 +27,8 @@ public class ArrayReader extends ValueReader
             if (p.hasToken(JsonToken.VALUE_NULL)) {
                 return null;
             }
-            throw JSONObjectException.from(p, "Unexpected token "+p.getCurrentToken()+"; should get START_ARRAY");
+            throw JSONObjectException.from(p, "Unexpected token %s; should get START_ARRAY",
+                    p.getCurrentToken());
         }
         
         CollectionBuilder b = r._collectionBuilder(null);
@@ -58,4 +62,4 @@ public class ArrayReader extends ValueReader
         } while (p.nextToken() != JsonToken.END_ARRAY);
         return b.buildArray(_elementType);
     }
-}
\ No newline at end of file
+}
