@@ -7,14 +7,18 @@ import java.util.*;
 
 import com.fasterxml.jackson.core.*;
 import com.fasterxml.jackson.core.io.SerializedString;
+import com.fasterxml.jackson.jr.ob.JSON;
 import com.fasterxml.jackson.jr.ob.JSONObjectException;
-import com.fasterxml.jackson.jr.ob.JSON.Feature;
+import com.fasterxml.jackson.jr.ob.api.ReaderWriterProvider;
+import com.fasterxml.jackson.jr.ob.api.ValueWriter;
 
-import static com.fasterxml.jackson.jr.ob.impl.TypeDetector.*;
+import static com.fasterxml.jackson.jr.ob.impl.ValueWriterLocator.*;
 
 /**
- * Object that handles serialization of simple Objects into
- * JSON.
+ * Object that handles serialization of simple Objects into underlying
+ * data format (usually JSON).
+ * Unlike {@link JSONReader}, writer does actually implement write methods itself
+ * and uses delegation for only some special cases.
  *<p>
  * Life-cycle is such that initial instance (called blueprint)
  * is constructed first (including possible configuration 
@@ -38,9 +42,9 @@ public class JSONWriter
     protected final boolean _writeNullValues;
 
     /**
-     * Object that is used to resolve types of values dynamically.
+     * Object that is used to dynamically find Bean (and custom type) value writers
      */
-    protected final TypeDetector _typeDetector;
+    protected final ValueWriterLocator _writerLocator;
 
     protected final TreeCodec _treeCodec;
 
@@ -64,11 +68,11 @@ public class JSONWriter
      * Constructor used for creating differently configured blueprint
      * instances
      */
-    public JSONWriter(int features, TypeDetector td, TreeCodec tc)
+    public JSONWriter(int features, ValueWriterLocator td, TreeCodec tc)
     {
         _features = features;
-        _writeNullValues = Feature.WRITE_NULL_PROPERTIES.isEnabled(features);
-        _typeDetector = td;
+        _writeNullValues = JSON.Feature.WRITE_NULL_PROPERTIES.isEnabled(features);
+        _writerLocator = td;
         _treeCodec = tc;
         _generator = null;
         _timezone = DEFAULT_TIMEZONE;
@@ -77,11 +81,11 @@ public class JSONWriter
     /**
      * Constructor for non-blueprint instances
      */
-    protected JSONWriter(JSONWriter base, int features, TypeDetector td, JsonGenerator g)
+    protected JSONWriter(JSONWriter base, int features, ValueWriterLocator td, JsonGenerator g)
     {
         _features = features;
-        _writeNullValues = Feature.WRITE_NULL_PROPERTIES.isEnabled(features);
-        _typeDetector = td;
+        _writeNullValues = JSON.Feature.WRITE_NULL_PROPERTIES.isEnabled(features);
+        _writerLocator = td;
         _treeCodec = base._treeCodec;
         _generator = g;
         _timezone = DEFAULT_TIMEZONE;
@@ -93,18 +97,32 @@ public class JSONWriter
     /**********************************************************************
      */
 
+    public JSONWriter withCacheCheck(int features) {
+//        int diff = (features ^ _features);
+
+        return this;
+    }
+
     public JSONWriter with(TreeCodec tc) {
         if (_treeCodec == tc) {
             return this;
         }
-        return _with(_features, _typeDetector, tc);
+        return _with(_features, _writerLocator, tc);
+    }
+
+    public JSONWriter with(ReaderWriterProvider rwp) {
+        ValueWriterLocator l = _writerLocator.with(rwp);
+        if (_writerLocator == l) {
+            return this;
+        }
+        return _with(_features, l, _treeCodec);
     }
 
     /**
      * Overridable method that all mutant factories call if a new instance
      * is to be constructed
      */
-    protected JSONWriter _with(int features, TypeDetector td, TreeCodec tc)
+    protected JSONWriter _with(int features, ValueWriterLocator td, TreeCodec tc)
     {
         if (getClass() != JSONWriter.class) { // sanity check
             throw new IllegalStateException("Sub-classes MUST override _with(...)");
@@ -124,7 +142,7 @@ public class JSONWriter
             throw new IllegalStateException("Sub-classes MUST override perOperationInstance(...)");
         }
         return new JSONWriter(this, features,
-                _typeDetector.perOperationInstance(features), g);
+                _writerLocator.perOperationInstance(this, features), g);
     }
 
     /*
@@ -133,25 +151,17 @@ public class JSONWriter
     /**********************************************************************
      */
 
+    /**
+     * Main entry point for non-blueprint instances: called for the root value to
+     * write it out.
+     */
     public void writeValue(Object value) throws IOException
     {
         if (value == null) {
             writeNullValue();
             return;
         }
-        _writeValue(value, _typeDetector.findSerializationType(value.getClass()));
-    }
-
-    @Deprecated // since 2.8
-    public void writeField(String fieldName, Object value) throws IOException
-    {
-        if (value == null) {
-            if (_writeNullValues) {
-                writeNullField(fieldName);
-            }
-            return;
-        }
-        writeField(fieldName, value, _typeDetector.findSerializationType(value.getClass()));
+        _writeValue(value, _writerLocator.findSerializationType(value.getClass()));
     }
 
     public void writeField(String fieldName, Object value, int type) throws IOException
@@ -259,10 +269,10 @@ public class JSONWriter
         }
 
         if (type < 0) { // Bean type!
-            BeanPropertyWriter[] props = _typeDetector.getPropertyWriters(type);
-            if (props != null) { // sanity check
+            ValueWriter writer = _writerLocator.getValueWriter(type);
+            if (writer != null) { // sanity check
                 _generator.writeFieldName(fieldName);
-                writeBeanValue(props, value);
+                writer.writeValue(this, _generator, value);
                 return;
             }
         }
@@ -373,10 +383,10 @@ public class JSONWriter
             return;
         }
 
-        if (type < 0) { // Bean type!
-            BeanPropertyWriter[] props = _typeDetector.getPropertyWriters(type);
-            if (props != null) { // sanity check
-                writeBeanValue(props, value);
+        if (type < 0) { // explicit ValueWriter
+            ValueWriter writer = _writerLocator.getValueWriter(type);
+            if (writer != null) { // sanity check
+                writer.writeValue(this, _generator, value);
                 return;
             }
         }
@@ -428,7 +438,7 @@ public class JSONWriter
                 _generator.writeNull();
                 continue;
             }
-            _writeValue(value, _typeDetector.findSerializationType(value.getClass()));
+            _writeValue(value, _writerLocator.findSerializationType(value.getClass()));
         }
         _generator.writeEndArray();
     }
@@ -454,7 +464,7 @@ public class JSONWriter
                     continue;
                 }
                 Class<?> cls = value.getClass();
-                int type = _typeDetector.findSerializationType(cls);
+                int type = _writerLocator.findSerializationType(cls);
                 writeField(key, value, type);
             }
         }
@@ -641,7 +651,7 @@ public class JSONWriter
     }
 
     protected void writeDateValue(Date v) throws IOException {
-        if (Feature.WRITE_DATES_AS_TIMESTAMP.isEnabled(_features)) {
+        if (JSON.Feature.WRITE_DATES_AS_TIMESTAMP.isEnabled(_features)) {
             writeLongValue(v.getTime());
         } else {
             writeStringValue(dateToString(v));
@@ -649,7 +659,7 @@ public class JSONWriter
     }
 
     protected void writeDateField(String fieldName, Date v) throws IOException {
-        if (Feature.WRITE_DATES_AS_TIMESTAMP.isEnabled(_features)) {
+        if (JSON.Feature.WRITE_DATES_AS_TIMESTAMP.isEnabled(_features)) {
             writeLongField(fieldName, v.getTime());
         } else {
             writeStringField(fieldName, dateToString(v));
@@ -657,7 +667,7 @@ public class JSONWriter
     }
 
     protected void writeEnumValue(Enum<?> v) throws IOException {
-        if (Feature.WRITE_ENUMS_USING_INDEX.isEnabled(_features)) {
+        if (JSON.Feature.WRITE_ENUMS_USING_INDEX.isEnabled(_features)) {
             writeIntValue(v.ordinal());
         } else {
             writeStringValue(v.toString());
@@ -665,14 +675,14 @@ public class JSONWriter
     }
 
     protected void writeEnumField(String fieldName, Enum<?> v) throws IOException {
-        if (Feature.WRITE_ENUMS_USING_INDEX.isEnabled(_features)) {
+        if (JSON.Feature.WRITE_ENUMS_USING_INDEX.isEnabled(_features)) {
             writeIntField(fieldName, v.ordinal());
         } else {
             writeStringField(fieldName, v.toString());
         }
     }
 
-    protected void writeBeanValue(BeanPropertyWriter[] props, Object bean) throws IOException
+    public void writeBeanValue(BeanPropertyWriter[] props, Object bean) throws IOException
     {
         _generator.writeStartObject();
         for (int i = 0, end = props.length; i < end; ++i) {
@@ -687,7 +697,7 @@ public class JSONWriter
             }
             int typeId = property.typeId;
             if (typeId == 0) {
-                typeId = _typeDetector.findSerializationType(value.getClass());
+                typeId = _writerLocator.findSerializationType(value.getClass());
             }
             _generator.writeFieldName(name);
             _writeValue(value, typeId);
@@ -707,9 +717,9 @@ public class JSONWriter
 
     protected void _checkUnknown(Object value) throws IOException
     {
-        if (Feature.FAIL_ON_UNKNOWN_TYPE_WRITE.isEnabled(_features)) {
+        if (JSON.Feature.FAIL_ON_UNKNOWN_TYPE_WRITE.isEnabled(_features)) {
             throw new JSONObjectException("Unrecognized type ("+value.getClass().getName()
-                    +"), don't know how to write (disable "+Feature.FAIL_ON_UNKNOWN_TYPE_WRITE
+                    +"), don't know how to write (disable "+JSON.Feature.FAIL_ON_UNKNOWN_TYPE_WRITE
                     +" to avoid exception)");
         }
     }
