@@ -18,6 +18,11 @@ public class JrsEmbeddedObject extends JrsValue.Scalar
         _value = v;
     }
 
+    @Override
+    public boolean isEmbeddedValue() {
+        return true;
+    }
+
     @Override
     public JsonToken asToken() {
         return JsonToken.VALUE_EMBEDDED_OBJECT;
